<?xml version="1.0" encoding="utf-8"?>
<!--

This stylesheet defines a parameter, qp:query-string, which takes an
optional string with the query part of a URI.  When this parameter is
given a string, the variable $qp:query-params is defined with an XML
encoding of the query parameters.
     
You can also decode any query string with the qp:decode-query-string
function.

Each ampersand-delimited parameter is encoded into XML with an element
in the qp namespace.  The localname of the element is the parameter
"key" - the part of the query parameter before the equal sign (if
any).  (If the key can't be represented as an XML element localname,
the parameter is omitted.)  If there is a parameter "value" - the part
after the first equal sign, if any - it appears as a text node under
the element; the element is empty otherwise.

Keys and values are "plus-decoded" ("+" becomes "%20"),
"percent-decoded" to sequences of UTF-8 octets (%21 becomes "!"), and
UTF-8 decoded to strings.

Elements with the same key may appear more than once.


To experiment, call this stylesheet with the initial template qp:main,
and supply a qp:query-string parameter.  (Note that in actual use,
there is no enclosing "result" element.  It is presented here to make
this example output less cluttered with namespace declarations on each
real element.)

  $ saxonb-xslt -xsl:query-params.xsl \
                -it:'{http://www.thoughtcrime.us/ns/xslt-query-params}main' \
		'{http://www.thoughtcrime.us/ns/xslt-query-params}query-string=ebat%c3%b5en%c3%a4oline-n%c3%a4ide=dead+beef+caf%c3%A9%21&boolean-param&repeating-param=foo&repeating-param=bar&non-unicode-heart=%3C3'
  <?xml version="1.0" encoding="UTF-8"?>
  <result xmlns="http://www.thoughtcrime.us/ns/xslt-query-params">
     <ebatõenäoline-näide>dead beef café!</ebatõenäoline-näide>
     <boolean-param/>
     <repeating-param>foo</repeating-param>
     <repeating-param>bar</repeating-param>
     <non-unicode-heart>&lt;3</non-unicode-heart>
  </result>


To use this file in your projects, xsl:import it and supply a
qp:query-string parameter with the query string of the "current
request" (whatever that means to you).  Or call the
qp:decode-query-string function.

-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xs="http://www.w3.org/2001/XMLSchema"
                xmlns:utf8="http://www.thoughtcrime.us/ns/xslt-utf8-decode"
                xmlns:qp="http://www.thoughtcrime.us/ns/xslt-query-params"
                version="2.0">
  <xsl:import href="utf8-decode.xsl"/>
  <xsl:output method="xml" indent="yes"/>
  
  <xsl:param name="qp:query-string" as="xs:string?"/>
  
  <!-- Outputs the query parameters as XML. -->
  <xsl:template name="qp:main">
    <xsl:text>&#10;&#10;</xsl:text>
    <xsl:comment> Note that in actual use, there is no enclosing "result" element.
It is presented here to make this example output less cluttered with
namespace declarations on each real element. </xsl:comment>
    <xsl:text>&#10;&#10;</xsl:text>
    <result xsl:exclude-result-prefixes="#all"
	    xmlns="http://www.thoughtcrime.us/ns/xslt-query-params">
      <xsl:namespace name=""
		     select="'http://www.thoughtcrime.us/ns/xslt-query-params'"/>
      <xsl:sequence select="$qp:query-params"/>
    </result>
  </xsl:template>
  
  <xsl:variable name="qp:query-params" as="element()*">
    <xsl:if test="exists ($qp:query-string)">
      <xsl:sequence select="qp:decode-query-string ($qp:query-string)"/>
    </xsl:if>
  </xsl:variable>
  
  <xsl:function name="qp:decode-query-string" as="element()*">
    <xsl:param name="query-string" as="xs:string"/>
    <xsl:variable name="query-pairs" as="xs:string*"
		  select="tokenize ($query-string, '&amp;')"/>
    <xsl:for-each select="$query-pairs">
      <xsl:variable name="key-e" as="xs:string" select="replace (., '=.*', '')"/>
      <xsl:variable name="value-e" as="xs:string?"
		    select="if (string-length (.) &gt; string-length ($key-e))
			    then substring (., string-length ($key-e) + 2)
			    else ()"/>
      <xsl:variable name="key" as="xs:string" select="qp:decode-uri-plus ($key-e)"/>
      <xsl:variable name="value" as="xs:string?" select="qp:decode-uri-plus ($value-e)"/>
      <!-- Avoid keys that would be illegal element names under XML. -->
      <xsl:if test="not (matches ($key, $qp:NCName-re, 's'))">
	<xsl:message select="concat ('No match: ', $key)"/>
	<xsl:message select="concat ('Name RE: ', $qp:NCName-re)"/>
      </xsl:if>
      <xsl:if test="matches ($key, $qp:NCName-re, 's')">
	<xsl:element namespace="http://www.thoughtcrime.us/ns/xslt-query-params"
		     name="{$key}" inherit-namespaces="no">
	  <xsl:sequence select="$value"/>
	</xsl:element>
      </xsl:if>
    </xsl:for-each>
  </xsl:function>
  
  <!-- c.f. http://www.w3.org/TR/REC-xml-names/#NT-NCName -->
  <xsl:variable name="qp:NCName-re" as="xs:string"
		select="concat ('^', $qp:NCNameStartChar-re, $qp:NCNameChar-re, '*$')"/>
  <xsl:variable name="qp:NCNameStartChar-re" as="xs:string"
		select="concat ('(', $qp:Letter-re, '|_)')"/>
  <xsl:variable name="qp:Letter-re" as="xs:string"
		select="concat ('(', $qp:BaseChar-re, '|', $qp:Ideographic-re, ')')"/>
  <xsl:variable name="qp:BaseChar-re" as="xs:string"
		select="
'[A-Za-z&#xC0;-&#xD6;&#xD8;-&#xF6;&#xF8;-&#xFF;&#x100;-&#x131;&#x134;-&#x13E;&#x141;-&#x148;&#x14A;-&#x17E;&#x180;-&#x1C3;&#x1CD;-&#x1F0;&#x1F4;-&#x1F5;&#x1FA;-&#x217;&#x250;-&#x2A8;&#x2BB;-&#x2C1;&#x386;&#x388;-&#x38A;&#x38C;&#x38E;-&#x3A1;&#x3A3;-&#x3CE;&#x3D0;-&#x3D6;&#x3DA;&#x3DC;&#x3DE;&#x3E0;&#x3E2;-&#x3F3;&#x401;-&#x40C;&#x40E;-&#x44F;&#x451;-&#x45C;&#x45E;-&#x481;&#x490;-&#x4C4;&#x4C7;-&#x4C8;&#x4CB;-&#x4CC;&#x4D0;-&#x4EB;&#x4EE;-&#x4F5;&#x4F8;-&#x4F9;&#x531;-&#x556;&#x559;&#x561;-&#x586;&#x5D0;-&#x5EA;&#x5F0;-&#x5F2;&#x621;-&#x63A;&#x641;-&#x64A;&#x671;-&#x6B7;&#x6BA;-&#x6BE;&#x6C0;-&#x6CE;&#x6D0;-&#x6D3;&#x6D5;&#x6E5;-&#x6E6;&#x905;-&#x939;&#x93D;&#x958;-&#x961;&#x985;-&#x98C;&#x98F;-&#x990;&#x993;-&#x9A8;&#x9AA;-&#x9B0;&#x9B2;&#x9B6;-&#x9B9;&#x9DC;-&#x9DD;&#x9DF;-&#x9E1;&#x9F0;-&#x9F1;&#xA05;-&#xA0A;&#xA0F;-&#xA10;&#xA13;-&#xA28;&#xA2A;-&#xA30;&#xA32;-&#xA33;&#xA35;-&#xA36;&#xA38;-&#xA39;&#xA59;-&#xA5C;&#xA5E;&#xA72;-&#xA74;&#xA85;-&#xA8B;&#xA8D;&#xA8F;-&#xA91;&#xA93;-&#xAA8;&#xAAA;-&#xAB0;&#xAB2;-&#xAB3;&#xAB5;-&#xAB9;&#xABD;&#xAE0;&#xB05;-&#xB0C;&#xB0F;-&#xB10;&#xB13;-&#xB28;&#xB2A;-&#xB30;&#xB32;-&#xB33;&#xB36;-&#xB39;&#xB3D;&#xB5C;-&#xB5D;&#xB5F;-&#xB61;&#xB85;-&#xB8A;&#xB8E;-&#xB90;&#xB92;-&#xB95;&#xB99;-&#xB9A;&#xB9C;&#xB9E;-&#xB9F;&#xBA3;-&#xBA4;&#xBA8;-&#xBAA;&#xBAE;-&#xBB5;&#xBB7;-&#xBB9;&#xC05;-&#xC0C;&#xC0E;-&#xC10;&#xC12;-&#xC28;&#xC2A;-&#xC33;&#xC35;-&#xC39;&#xC60;-&#xC61;&#xC85;-&#xC8C;&#xC8E;-&#xC90;&#xC92;-&#xCA8;&#xCAA;-&#xCB3;&#xCB5;-&#xCB9;&#xCDE;&#xCE0;-&#xCE1;&#xD05;-&#xD0C;&#xD0E;-&#xD10;&#xD12;-&#xD28;&#xD2A;-&#xD39;&#xD60;-&#xD61;&#xE01;-&#xE2E;&#xE30;&#xE32;-&#xE33;&#xE40;-&#xE45;&#xE81;-&#xE82;&#xE84;&#xE87;-&#xE88;&#xE8A;&#xE8D;&#xE94;-&#xE97;&#xE99;-&#xE9F;&#xEA1;-&#xEA3;&#xEA5;&#xEA7;&#xEAA;-&#xEAB;&#xEAD;-&#xEAE;&#xEB0;&#xEB2;-&#xEB3;&#xEBD;&#xEC0;-&#xEC4;&#xF40;-&#xF47;&#xF49;-&#xF69;&#x10A0;-&#x10C5;&#x10D0;-&#x10F6;&#x1100;&#x1102;-&#x1103;&#x1105;-&#x1107;&#x1109;&#x110B;-&#x110C;&#x110E;-&#x1112;&#x113C;&#x113E;&#x1140;&#x114C;&#x114E;&#x1150;&#x1154;-&#x1155;&#x1159;&#x115F;-&#x1161;&#x1163;&#x1165;&#x1167;&#x1169;&#x116D;-&#x116E;&#x1172;-&#x1173;&#x1175;&#x119E;&#x11A8;&#x11AB;&#x11AE;-&#x11AF;&#x11B7;-&#x11B8;&#x11BA;&#x11BC;-&#x11C2;&#x11EB;&#x11F0;&#x11F9;&#x1E00;-&#x1E9B;&#x1EA0;-&#x1EF9;&#x1F00;-&#x1F15;&#x1F18;-&#x1F1D;&#x1F20;-&#x1F45;&#x1F48;-&#x1F4D;&#x1F50;-&#x1F57;&#x1F59;&#x1F5B;&#x1F5D;&#x1F5F;-&#x1F7D;&#x1F80;-&#x1FB4;&#x1FB6;-&#x1FBC;&#x1FBE;&#x1FC2;-&#x1FC4;&#x1FC6;-&#x1FCC;&#x1FD0;-&#x1FD3;&#x1FD6;-&#x1FDB;&#x1FE0;-&#x1FEC;&#x1FF2;-&#x1FF4;&#x1FF6;-&#x1FFC;&#x2126;&#x212A;-&#x212B;&#x212E;&#x2180;-&#x2182;&#x3041;-&#x3094;&#x30A1;-&#x30FA;&#x3105;-&#x312C;&#xAC00;-&#xD7A3;]'"/>
  <xsl:variable name="qp:Ideographic-re" as="xs:string"
		select="'[&#x4E00;-&#x9FA5;&#x3007;&#x3021;-&#x3029;]'"/>
  <xsl:variable name="qp:NCNameChar-re" as="xs:string"
		select="'[A-Z_a-z&#xC0;-&#xD6;&#xD8;-&#xF6;&#xF8;-&#x2FF;&#x370;-&#x37D;&#x37F;-&#x1FFF;&#x200C;-&#x200D;&#x2070;-&#x218F;&#x2C00;-&#x2FEF;&#x3001;-&#xD7FF;&#xF900;-&#xFDCF;&#xFDF0;-&#xFFFD;&#x10000;-&#xEFFFF;.0-9&#xB7;&#x0300;-&#x036F;&#x203F;-&#x2040;-]'"/>
  
  <!-- Decodes a percent/plus-encoded string. -->
  <xsl:function name="qp:decode-uri-plus" as="xs:string?">
    <xsl:param name="encoded" as="xs:string?"/>
    <xsl:if test="exists ($encoded)">
      <xsl:value-of select="qp:decode-uri (replace ($encoded, '\+', '%20'))"/>
    </xsl:if>
  </xsl:function>
  
  <!-- Decodes a percent-encoded string. -->
  <xsl:function name="qp:decode-uri" as="xs:string?">
    <xsl:param name="encoded" as="xs:string?"/>
    <xsl:if test="exists ($encoded)">
      <xsl:variable name="decoded-as-utf-8" as="xs:integer*">
	<xsl:analyze-string select="$encoded" regex="%([0-9a-f]{{2}})" flags="i">
          <xsl:matching-substring>
            <xsl:sequence select="qp:decode-hex (regex-group (1))"/>
          </xsl:matching-substring>
          <xsl:non-matching-substring>
	    <!-- Below is U+FFFD REPLACEMENT CHARACTER in UTF-8. -->
	    <xsl:sequence select="for $cp in string-to-codepoints (.)
				  return if ($cp &lt; 128) then $cp else (239, 191, 189)"/>
          </xsl:non-matching-substring>
	</xsl:analyze-string>
      </xsl:variable>
      <xsl:sequence select="utf8:decode-string ($decoded-as-utf-8)"/>
    </xsl:if>
  </xsl:function>
  
  <!-- Decodes a string expressing a non-negative integer in
       hexadecimal.  There is no handling of the error case of an
       invalid string being given. -->
  <xsl:function name="qp:decode-hex" as="xs:integer">
    <xsl:param name="hex" as="xs:string"/>
    <xsl:variable name="length" as="xs:integer" select="string-length ($hex)"/>
    <xsl:variable name="digit-val" as="xs:integer"
		  select="string-length (substring-before ('0123456789ABCDEF',
			                                   upper-case (substring ($hex, $length, 1))))"/>
    <xsl:choose>
      <xsl:when test="$length = 1">
	<xsl:sequence select="$digit-val"/>
      </xsl:when>
      <xsl:otherwise>
	<xsl:sequence select="qp:decode-hex (substring ($hex, 1, $length - 1)) * 16 +
			      $digit-val"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>
</xsl:stylesheet>
